#version 330 core

// Interpolated values from the vertex shaders
in vec2 UV;
in vec4 ShadowCoord;
in vec3 Position_worldspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;

in vec3 LightDirection_tangentspace;
in vec3 EyeDirection_tangentspace;

// Ouput data
out vec3 color;

// Values that stay constant for the whole mesh.
uniform sampler2D DiffuseTextureSampler;
uniform sampler2D NormalTextureSampler;
uniform sampler2D SpecularTextureSampler;
uniform sampler2D ShadowMap;
uniform mat4 V;
uniform mat4 M;
uniform mat3 MV3x3;
uniform vec3 LightPosition_worldspace;  
uniform vec3 CameraPosition;    

vec4 ShadowCoordPostW;

// FOG

uniform vec3 FogColor = vec3(0.45, 0.59, 0.68);
const float FogDensity = 0.05;

// END FOG

float linstep(float min, float max, float v) {
	return clamp((v-min)/(max-min), 0.0, 1.0);
}

float chebyshevUpperBound( float distance) {
	//We retrieve the two moments previously stored (depth and depth*depth)
	vec2 moments = texture2D(ShadowMap,ShadowCoordPostW.xy).rg;
	
	// Surface is fully lit. as the current fragment is before the light occluder
	if (distance <= moments.x)
		return 1.0 ;
	
	// The fragment is either in shadow or penumbra. We now use chebyshev's upperBound to check
	// How likely this pixel is to be lit (p_max)
	float variance = moments.y - (moments.x*moments.x);
	variance = max(variance,0.0000002);
	
	float d = distance - moments.x;
	float p_max = linstep(0.2, 1.0, variance / (variance + d*d));
	
	return min(p_max, 1.0);
}

void main(){
	// FOG
	vec3 dist = Position_worldspace - CameraPosition;
	float distf = dot(dist,dist)*0.01;
	float fogFactor = 1.0 /exp( (distf * FogDensity)* (distf * FogDensity));
	fogFactor = clamp( fogFactor, 0.0, 1.0 );
	if (fogFactor < 0.001) {
		// Don't care about the rest, won't see anyway...
		color = FogColor;
		return;
	}
	// Light emission properties
	// You probably want to put them as uniforms
	vec3 LightColor = vec3(1,1,1);
	float LightPower = 80.0;
	
	// Material properties
	vec3 MaterialDiffuseColor = texture2D( DiffuseTextureSampler, UV ).rgb;
	vec3 MaterialAmbientColor = vec3(0.2,0.2,0.2) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = texture2D( SpecularTextureSampler, UV ).rgb * 0.3;

	// Local normal, in tangent space. V tex coordinate is inverted because normal map is in TGA (not in DDS) for better quality
	vec3 TextureNormal_tangentspace = normalize(texture2D( NormalTextureSampler, UV ).rgb*2.0 - 1.0);
	//vec2(UV.x,-UV.y)

	// Distance to the light
	float distance = length( LightPosition_worldspace - Position_worldspace );

	// Normal of the computed fragment, in camera space
	vec3 n = TextureNormal_tangentspace;
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize(LightDirection_tangentspace);
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot( n,l ), 0,1 );

	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_tangentspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	
	// fixed bias
	//float bias = 0.005;
	// calculate visibility in shadow map
	//float visibility = textureProj(ShadowMap, ShadowCoord, bias);
	ShadowCoordPostW = ShadowCoord / ShadowCoord.w;
	float shadow = chebyshevUpperBound(ShadowCoordPostW.z);
	
	color = 
		// Ambient : simulates indirect lighting
		MaterialAmbientColor +
		// Diffuse : "color" of the object
		shadow * MaterialDiffuseColor * LightColor * LightPower * cosTheta / (distance*distance) +
		// Specular : reflective highlight, like a mirror
		shadow * MaterialSpecularColor * LightColor * LightPower * pow(cosAlpha,2) / (distance*distance);
		
	
	
	// mix fog
	color = mix(FogColor, color, fogFactor);
}